@@ -4,14 +4,12 @@ from pythonagent.lib import get_ident
 import ctypes
 import os
 
-
 from pythonagent.agent.probes.havoc.havoc_manager import NDNetHavocMonitor, NDHavocException
 
 from pythonagent.agent.internal import udp
 from pythonagent.agent.probes.frameworks import wsgi  # aiflagset
 from pythonagent.agent.probes.Instrumentation.find import find
-
-
+from pythonagent.utils import generate_flow_path_id, get_utf8_bytes
 
 
 class TransactionContext(object):
@@ -30,6 +28,7 @@ class TransactionContext(object):
         self.server_name = None
         self.instance_name = None
         self.url_path = None
+        self.flow_path_id = None
 
 
 class Agent(object):
@@ -100,9 +99,7 @@ class Agent(object):
             self.instance_name = "default"
             self.function_name = "default"
 
-
         self.start()
-
         if self.cav_env == "NATIVE":
             self.ai_wrapper_begin()
             self.ad_wrapper()
@@ -115,7 +112,7 @@ class Agent(object):
         self.havoc_monitor = NDNetHavocMonitor.get_instance()
 
     def start(self):
-        path = '/usr/local/lib/libndsdk.so'
+        path = '/usr/local/lib/libndsdk_py.so'
 
         if self.cav_env == "NATIVE":
             self.lib = self.load_common_lib(path)
@@ -154,7 +151,7 @@ class Agent(object):
         self.lib.nd_method_entry.argtypes = [ctypes.c_void_p, ctypes.c_char_p]
         self.lib.nd_method_exit.argtypes = [ctypes.c_void_p, ctypes.c_char_p]
 
-        self.lib.nd_bt_begin.argtypes = [ctypes.c_char_p, ctypes.c_char_p]
+        self.lib.nd_bt_begin.argtypes = [ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p]
         self.lib.nd_bt_begin.restype = ctypes.c_void_p
 
         self.lib.nd_bt_end.argtypes = [ctypes.c_void_p, ctypes.c_int]
@@ -193,11 +190,46 @@ class Agent(object):
         thd_ftype = ctypes.CFUNCTYPE(ctypes.c_char_p)
         self.c_thd = thd_ftype(self.thd)
 
+        self.lib.sdk_httpReqRespWrapper.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_int]
+
+        #self.lib.sdk_get_otl_traceparent.argtypes = [ctypes.c_void_p]
+        #self.lib.sdk_get_otl_traceparent.restype = ctypes.c_char_p
+
+        #self.lib.sdk_get_otl_tracestate.argtypes = [ctypes.c_void_p]
+        #self.lib.sdk_get_otl_tracestate.restype = ctypes.c_char_p
+        
         self.logger.info("load common lib -> lib.obj :{0}".format(self.lib))
+        
         return self.lib
 
+
+    def get_trace_headers(self, bt):
+        traceparent = self.lib.sdk_get_otl_traceparent(bt)
+        tracestate = self.lib.sdk_get_otl_tracestate(bt)
+        return traceparent, tracestate
+
+
+    def http_req_resp_wrapper_native(self, bt, rbuffer, rtype, statuscode):
+        rbuffer = str(rbuffer)
+        rbuffer = bytes(rbuffer, 'utf-8')
+        r_buffer_c = ctypes.c_char_p(rbuffer)
+
+        rtype = str(rtype)
+        rtype = bytes(rtype, 'utf-8')
+        r_type_c = ctypes.c_char_p(rtype)
+
+        self.lib.sdk_httpReqRespWrapper(bt, r_buffer_c, r_type_c, statuscode)
+
+
+    def http_req_resp_wrapper(self, bt, rbuffer, rtype, statuscode):
+        if self.cav_env == "NATIVE":
+            self.http_req_resp_wrapper_native(bt, rbuffer, rtype, statuscode)
+        elif self.cav_env == "AWS_LAMBDA":
+            udp.http_req_resp_wrapper(self, bt, rbuffer, rtype, statuscode)
+
+
     def ai_wrapper_begin_native(self):
-        self.logger.info("************************ callback registered ********************************************")
+        print("************************ callback registered ********************************************")
         self.lib.register_AI_start_callback(self.c_startaisession)
 
     def ai_wrapper_begin(self):
@@ -216,13 +248,15 @@ class Agent(object):
             udp.ai_wrapper_end()
 
     def endaisessioncc(self, char):
-        self.logger.info("INSIDE END AI SESSION")
+        print("INSIDE END AI SESSION")
 
     def startaisession(self, istart, startTime, duration, sname):
-        self.logger.info("session status \n", istart)
-        self.logger.info(" IN startaisession", sname.decode('ascii'))
+        print("session status \n", istart)
+        print(" IN startaisession", sname.decode('ascii'))
         wsgi.aiflagset(istart, startTime, duration, sname)
-        pathd = "/opt/cavisson/netdiagnostics/logs/"
+        #pathd = "/opt/cavisson/netdiagnostics/logs/"
+        nd_home = os.environ.get('ND_HOME')
+        pathd = nd_home + '/logs/'
         filenameai1 = str(self.sess.decode('ascii')) + ".txt"
         aifile_to_open_to_d = os.path.join(pathd, filenameai1)
         self.lib.handleUploadDownload(aifile_to_open_to_d)
@@ -236,7 +270,7 @@ class Agent(object):
 
 
     def thd(self):
-        self.logger.info("INSIDE THD")
+        print("INSIDE THD")
         # dumpstacks()
 
     def thd_funct(self, a, b, c, d):
@@ -245,7 +279,9 @@ class Agent(object):
     # @profile
     def sdk_init_native(self):
         self.logger.info("sdk_init -> lib.obj :{0}".format(self.lib))
-        self.lib.nd_init("python", "cavlib")
+        agent_type = bytes("python", 'utf-8')
+        log_file_name = bytes("cavlib", 'utf-8')
+        self.lib.nd_init2(agent_type, log_file_name)
 
     def sdk_init(self):
         if self.cav_env == "NATIVE":
@@ -266,7 +302,7 @@ class Agent(object):
 
     # @profile
     def method_entry_native(self, bt, method):
-        self.logger.info('Inside method entry {} {}'.format( method, bt))
+        print('Inside method entry ', method, bt)
         self.logger.info("method entry -> lib.obj :{0}".format(self.lib))
         self.logger.info("method entry -> method name :{0}".format(method))
         # defer C.free(unsafe.Pointer(method_c))
@@ -285,17 +321,15 @@ class Agent(object):
             udp.method_entry(self, bt, method, "NA", "NA")
 
     def method_entry_http_callout(self, bt, method, query_string, url_parameter):
-
-        self.havoc_monitor.apply_outbound_service_delay(query_string)
-
         if self.cav_env == "NATIVE":
             pass
         elif self.cav_env == "AWS_LAMBDA":
             udp.method_entry(self, bt, method, query_string, url_parameter)
+        self.havoc_monitor.apply_outbound_service_delay(query_string)
 
     #@profile
     def method_exit_native(self, bt, method):
-            self.logger.info('in method exit {} {}'.format(method, bt))
+            print('in method exit', method, bt)
             self.logger.info("load common lib -> lib.obj :{0}".format(self.lib))
             self.logger.info("method entry -> method name :{0}".format(method))
             # defer C.free(unsafe.Pointer(method_c))
@@ -304,7 +338,7 @@ class Agent(object):
                 method_bytes = bytes(method, 'utf-8')
             except:
                 method_bytes = bytes(method.encode("utf-8"))
-            self.logger.info("\n\n", method_bytes)
+            print("\n\n", method_bytes)
             self.lib.nd_method_exit(bt, ctypes.c_char_p(method_bytes))
 
     def method_exit(self, bt, method, status=200, duration=0):
@@ -334,14 +368,9 @@ class Agent(object):
     def start_business_transaction_native(self, bt_name, correlation_header):
         if os.environ['nd_init_done'] == '0':
             self.start()
-            self.logger.info("start_business_transaction ->")
+            print("start_business_transaction ->")
             self.logger.debug(" start_business_transaction -> lib.obj {0}:".format(self.lib))
-            # btname = bytes(bt_name, 'utf-8')
-            # self.logger.info("btname type and value: {0} and {1}".format(type(btname), btname))
-            # bt_name_c = ctypes.c_char_p(btname)
-            # self.logger.info("bt_name_c value: {0}".format(bt_name_c))
-            # correlation_header = bytes(correlation_header, 'utf-8')
-            # correlation_header_c = ctypes.c_char_p(correlation_header)
+
         try:
             # self.logger.debug(" start_business_transaction -> lib.obj {0}:".format(self.lib))
             btname = bytes(bt_name, 'utf-8')
@@ -360,47 +389,44 @@ class Agent(object):
             correlation_header = bytes(correlation_header.encode("utf-8"))
             correlation_header_c = ctypes.c_char_p(correlation_header)
 
-        bt = self.lib.nd_bt_begin(bt_name_c, correlation_header_c)
+        self.get_transaction_context().flow_path_id = generate_flow_path_id() 
+        
+        nd_cookie_c = ctypes.c_char_p(None)
+        nv_cookie_c = ctypes.c_char_p(None)
+        bt_header_value_c = ctypes.c_char_p(None)
+        
+        flow_path_id = self.get_transaction_context().flow_path_id
+        flow_path_id_bytes = get_utf8_bytes(str(flow_path_id))
+        flow_path_id_c =  ctypes.c_char_p(flow_path_id_bytes)
+
+        bt = self.lib.nd_bt_begin(bt_name_c, correlation_header_c, nd_cookie_c, nv_cookie_c, bt_header_value_c, flow_path_id_c)
         #self.logger.info("bt datatype : {0}".format(type(bt)))
-        self.logger.info("btname value: {0}".format(btname))
+        print("btname value: {0}".format(btname))
         # print('-----------------\nCurrent heap memory stack for BT: ',bt_name,' is : ',hpy().heap())
         self.active_bts.add(bt)
         self.set_current_bt(bt)
-        self.logger.info("BT value of this txn: {}".format(bt))
+        print("BT value of this txn: ", bt)
         return bt
 
     def start_business_transaction(self, bt_name, correlation_header):
         bt = None
+        if self.cav_env == "NATIVE":
+            bt = self.start_business_transaction_native(bt_name, correlation_header)
+        elif self.cav_env == "AWS_LAMBDA":
+            bt = udp.start_business_transaction(self, bt_name, correlation_header)
 
-
-        """
         try:
             self.havoc_monitor.apply_inbound_service_delay(bt_name)
         except Exception as e:
             # Will fail first time
             print(e)
             pass
-        """
-
-        if self.cav_env == "NATIVE":
-            bt = self.start_business_transaction_native(bt_name, correlation_header)
-        elif self.cav_env == "AWS_LAMBDA":
-            bt = udp.start_business_transaction(self, bt_name, correlation_header)
-
-
-
-        try:
-            self.havoc_monitor.apply_inbound_service_delay(bt_name)
-        except Exception as e:
-            # Will fail first time
-            self.logger.info("{}".format(e))
-            pass
 
         return bt
 
     # @profile
     def end_business_transaction_native(self, bt):
-        self.logger.info("end_business_transaction--> BT: {}".format(bt))
+        print("end_business_transaction--> BT: ", bt)
         self.logger.info('end_business_transaction--> lib variable in end txn: {0}'.format(self.lib))
         self.logger.info('At end_business_transaction bt value is {0}'.format(bt))
         statuscode = self.get_current_status_code()
@@ -456,7 +482,8 @@ class Agent(object):
 
     def db_call_begin(self, bt, db_host, db_query):
         if self.cav_env == "NATIVE":
-            self.db_call_begin_native(bt, db_host, db_query)
+            ip_handle = self.db_call_begin_native(bt, db_host, db_query)
+            return ip_handle
         elif self.cav_env == "AWS_LAMBDA":
             udp.db_call_begin(bt, db_host, db_query)
 
@@ -541,10 +568,10 @@ class Agent(object):
 
 
     def wait_for_end(self, timeout_ms=None):
-        """Wait for the printagent to finish reporting any pending BTs.
+        """Wait for the agent to finish reporting any pending BTs.
 
         """
-        self.tx_svc.waitprint_for_end(timeout_ms=timeout_ms)
+        self.tx_svc.wait_for_end(timeout_ms=timeout_ms)
 
     #@property
     def enabled(self):
@@ -553,7 +580,7 @@ class Agent(object):
             """
             return (
                     self.nd_init_done and
-                    selfprint.config_svc and self.config_svc.enabled and
+                    self.config_svc and self.config_svc.enabled and
                     self.tx_svc)
 
     def get_current_bt(self):
@@ -592,13 +619,12 @@ class Agent(object):
         self.current_status_code.pop(get_ident(), None)
     
     def set_transaction_context(self, context):
-
-        #print("INVOKED set_transaction_context ", context)
         if context:
             context.tier_name = self.tier_name
             context.server_name = self.server_name
             context.instance_name = self.instance_name
             context.function_name = self.function_name
+            context.flow_path_id = generate_flow_path_id()
             self.transaction_context[get_ident()] = context
     
     def get_transaction_context(self):
@@ -625,8 +651,3 @@ class AgentNotReadyException(Exception):
 class IgnoreTransaction(Exception):
     pass
 
-
-
-
-
-
