@@ -15,7 +15,6 @@ class Header(object):
         self.apiReqId = apiReqId
         self.awsReqId = awsReqId
         self.funcName = funcName
-
         self.tags = "tierName={};ndAppServerHost={};appName={}".format(context.tier_name, context.server_name, funcName)
 
         self.agentType = 0  # Python
@@ -56,7 +55,18 @@ class Header(object):
         self.funcNameLen = len(self.funcName)
         self.tagslength = len(self.tags)
 
-        self.whLen = 20 + 2 + 2 + self.apiReqLen + self.awsReqLen + self.funcNameLen + self.tagslength + 1
+        self.whLen = (1                   # ^ (1c)
+                      + 4                 # whLen(int)
+                      + 4                 # apiReqLen(int)
+                      + 4                 # awsReqLen(int)
+                      + 4                 # funcNameLen(int)
+                      + 4                 # tagslength(int)
+                      + 2                 # agentType(short)
+                      + 2                 # messageType(short)
+                      + self.apiReqLen    # variable
+                      + self.awsReqLen    # variable
+                      + self.funcNameLen  # variable
+                      + self.tagslength)  # variable
 
         header_struct = struct.Struct('=ciiiiihh{}s{}s{}s{}s'.format(self.apiReqLen, self.awsReqLen,
                                                                      self.funcNameLen, self.tagslength))
@@ -81,13 +91,16 @@ class Header(object):
 
 class StartTransactionMessage(object):
 
-    def __init__(self, url):
+    def __init__(self, url, flow_path_instance):
         self.fp_header = "dummy_fp_header"
         self.url = url
         self.btHeaderValue = "dummy_btHeaderValue"
         self.ndCookieSet = ""
         self.nvCookieSet = ""
         self.correlationHeader = "dummy_correlationHeader"
+        self.flow_path_instance = flow_path_instance
+        self.qTimeMS = 0  # DUMMY
+        self.startTimeFP = 0  # DUMMY
 
     def encode(self):
 
@@ -98,19 +111,38 @@ class StartTransactionMessage(object):
         self.len_nvCookieSet = len(self.nvCookieSet)
         self.len_correlationHeader = len(self.correlationHeader)
 
-        self.flowpathinstance = 0 # DUMMY
-        self.qTimeMS = 0 # DUMMY
-        self.startTimeFP = 0 # DUMMY
 
-        # self.header_len = 48
-        # self.total_len = self.header_len + self.len_url + 3
+        self.header_len = (4                            # header_len(int)
+                           + 4                          # total_len(int)
+                           + 4)                         # msg_type(int)
 
-        self.header_len = 12
-        self.total_len = 12 + 48 + self.len_fp_header + self.len_url + self.len_btHeaderValue + self.len_ndCookieSet + self.len_nvCookieSet + self.len_correlationHeader + 3
+        self.total_len = (1                             # ^ (1c)
+                          + 4                           # header_len(int)
+                          + 4                           # total_len(int)
+                          + 4                           # msg_type(int)
+                          + 1                           # | (1c)
+                          + 4                           # len_fp_header(int)
+                          + 4                           # len_url(int)
+                          + 4                           # len_btHeaderValue(int)
+                          + 4                           # len_ndCookieSet(int)
+                          + 4                           # len_nvCookieSet(int)
+                          + 4                           # len_correlationHeader(int)
+                          + 8                           # flow_path_instance(long long)
+                          + 8                           # qTimeMS(long)
+                          + 8                           # startTimeFP(long long)
+                          + self.len_fp_header          # variable
+                          + self.len_url                # variable
+                          + self.len_btHeaderValue      # variable
+                          + self.len_ndCookieSet        # variable
+                          + self.len_nvCookieSet        # variable
+                          + self.len_correlationHeader  # variable
+                          + 1)                          # \n (1c)
 
         self.msg_type = 2  # START TRANSACTION
 
-        format_str = "=ciiiciiiiiiqqq{}s{}s{}s{}s{}s{}sc".format(self.len_fp_header, self.len_url, self.len_btHeaderValue, self.len_ndCookieSet, self.len_nvCookieSet, self.len_correlationHeader)
+        format_str = "=ciiiciiiiiiqqq{}s{}s{}s{}s{}s{}sc".format(self.len_fp_header, self.len_url,
+                                                                 self.len_btHeaderValue, self.len_ndCookieSet,
+                                                                 self.len_nvCookieSet, self.len_correlationHeader)
 
         start_transaction_message_struct = struct.Struct(format_str)
 
@@ -125,7 +157,7 @@ class StartTransactionMessage(object):
                                             self.len_ndCookieSet,
                                             self.len_nvCookieSet,
                                             self.len_correlationHeader,
-                                            self.flowpathinstance,
+                                            self.flow_path_instance,
                                             self.qTimeMS,
                                             self.startTimeFP,
                                             self.fp_header.encode('utf-8'),
@@ -143,29 +175,45 @@ class StartTransactionMessage(object):
 
 class MethodEntryMessage(object):
 
-    def __init__(self, methodName, query_string, url_parameter):
+    def __init__(self, methodName, query_string, url_parameter, flow_path_instance):
         self.methodName = methodName
         self.query_string = query_string
         self.urlParameter = url_parameter
 
-    def encode(self):
         self.mid = 0  # DUMMY
-        self.flowpathinstance = 0  # DUMMY
-        self.threadId = 0 #threading.get_ident() # DUMMY
+        self.flow_path_instance = flow_path_instance
+        self.threadId = 0  # DUMMY
         self.startTime = 0  # DUMMY
+
+    def encode(self):
+
         self.len_methodName = len(self.methodName)
         self.len_query_string = len(self.query_string)
         self.len_urlParameter = len(self.urlParameter)
 
-        self.header_len = 12
-        self.total_len = 12 + 40 + self.len_methodName + self.len_query_string + self.len_urlParameter + 3
-        #self.total_len = 12 + 48 + self.len_methodName + self.len_query_string + self.len_urlParameter + 3
+        self.header_len = (4                            # header_len(int)
+                           + 4                          # total_len(int)
+                           + 4)                         # msg_type(int)
 
+        self.total_len = (1                             # ^ (1c)
+                          + 4                           # header_len(int)
+                          + 4                           # total_len(int)
+                          + 4                           # msg_type(int)
+                          + 1                           # | (1c)
+                          + 4                           # mid(int)
+                          + 8                           # flow_path_instance(long long)
+                          + 8                           # threadId(long)
+                          + 8                           # startTime(long long)
+                          + 4                           # len_methodName(int)
+                          + 4                           # len_query_string(int)
+                          + 4                           # len_urlParameter(int)
+                          + self.len_methodName         # variable
+                          + self.len_query_string       # variable
+                          + self.len_urlParameter       # variable
+                          + 1)                          # \n (1c)
 
         self.msg_type = 0  # METHOD ENTRY
-        # method_entry_message_struct = struct.Struct("=ciiiciqqqiii{}sc".format(self.len_methodName))
 
-        
         format_str = "=ciiiciqqqiii{}s{}s{}sc".format(self.len_methodName, self.len_query_string, self.len_urlParameter)
 
         method_entry_message_struct = struct.Struct(format_str)
@@ -243,7 +291,7 @@ class MethodExitMessage(object):
                           + 4                                      # isCallout(int)
                           + 8                                      # threadId(long)
                           + 8                                      # duration(long)
-                          + 8                                      # flowpathinstance(long long)
+                          + 8                                      # flow_path_instance(long long)
                           + 8                                      # cpuTime(long long)
                           + 4                                      # len_methodName(int)
                           + 4                                      # len_backend_header(int)
@@ -271,7 +319,7 @@ class MethodExitMessage(object):
                                       self.isCallout,
                                       self.threadId,
                                       self.duration,
-                                      self.flowpathinstance,
+                                      self.flow_path_instance,
                                       self.cpuTime,
                                       self.len_methodName,
                                       self.len_backend_header,
@@ -291,10 +339,10 @@ class MethodExitMessage(object):
 
 class EndTransactionMessage(object):
 
-    def __init__(self, status_code):
+    def __init__(self, status_code, flow_path_instance):
 
         self.status_code = status_code
-        self.flow_path_instance = 0  # DUMMY
+        self.flow_path_instance = flow_path_instance
         self.endTime = 0  # DUMMY
         self.cpuTime = 0  # DUMMY
         self.header_len = None
@@ -340,20 +388,19 @@ class EndTransactionMessage(object):
 
 class TransactionEncodeHttpMessage(object):
 
-    def __init__(self, bt, http_host, url):
+    def __init__(self, bt, rbuffer, rtype, statuscode, flow_path_instance):
 
         # bt = 140544547110928
         # http_host = "www.google.com"
         # url = "/"
-        self.buffer = "dummy_buffer"
-        self.type = "req"  # or "resp"
-
+        self.buffer = rbuffer
+        self.type = rtype
+        self.statuscode = statuscode
+        self.flow_path_instance = flow_path_instance
 
     def encode(self):
-        self.statuscode = 0  # DUMMY
         self.buffer_len = len(self.buffer)
         self.type_len = len(self.type)
-        self.flowpathinstance = 0  # DUMMY
 
         self.header_len = 12
         self.total_len = 12 + 20 + self.buffer_len + self.type_len + 3
@@ -371,7 +418,7 @@ class TransactionEncodeHttpMessage(object):
                                                   self.statuscode,
                                                   self.buffer_len,
                                                   self.type_len,
-                                                  self.flowpathinstance,
+                                                  self.flow_path_instance,
                                                   self.buffer.encode('utf-8'),
                                                   self.type.encode('utf-8'),
                                                   bytes('\n', "utf-8"))
@@ -408,7 +455,7 @@ def create_start_transaction_message(context, bt_name, correlation_header):
     header = Header(context, context.api_request_id, context.aws_request_id, context.function_name)
     header_packet = header.encode()
 
-    start_transaction_message = StartTransactionMessage(bt_name)
+    start_transaction_message = StartTransactionMessage(bt_name, context.flow_path_id)
     start_transaction_message_packet = start_transaction_message.encode()
 
     full_packet = header_packet + start_transaction_message_packet
@@ -421,7 +468,7 @@ def create_method_entry_message(context, bt, method, query_string, url_parameter
     header = Header(context, context.api_request_id, context.aws_request_id, context.function_name)
     header_packet = header.encode()
 
-    method_entry_message = MethodEntryMessage(method, query_string, url_parameter)
+    method_entry_message = MethodEntryMessage(method, query_string, url_parameter, context.flow_path_id)
     method_entry_message_packet = method_entry_message.encode()
 
     full_packet = header_packet + method_entry_message_packet
@@ -433,7 +480,7 @@ def create_method_exit_message(context, bt, method, backend_header, status, dura
     header = Header(context, context.api_request_id, context.aws_request_id, context.function_name)
     header_packet = header.encode()
 
-    method_exit_message = MethodExitMessage(method, backend_header, status, duration)
+    method_exit_message = MethodExitMessage(method, backend_header, status, duration, context.flow_path_id)
     method_exit_message_packet = method_exit_message.encode()
 
     full_packet = header_packet + method_exit_message_packet
@@ -445,10 +492,10 @@ def create_end_transaction_message(context, bt, statuscode):
     header = Header(context, context.api_request_id, context.aws_request_id, context.function_name)
     header_packet = header.encode()
 
-    start_transaction_message = EndTransactionMessage(statuscode)
-    start_transaction_message_packet = start_transaction_message.encode()
+    end_transaction_message = EndTransactionMessage(statuscode, context.flow_path_id)
+    end_transaction_message_packet = end_transaction_message.encode()
 
-    full_packet = header_packet + start_transaction_message_packet
+    full_packet = header_packet + end_transaction_message_packet
 
     return full_packet
 
@@ -467,11 +514,11 @@ def create_havoc_message(context, havoc_header):
     return full_packet
 
 
-def create_transaction_encode_http_message(context, bt, http_host, url):
-    header = Header(context.api_request_id, context.aws_request_id, context.function_name)
+def create_transaction_encode_http_message(context, bt, rbuffer, rtype, statuscode):
+    header = Header(context, context.api_request_id, context.aws_request_id, context.function_name)
     header_packet = header.encode()
 
-    transaction_encode_http_message = TransactionEncodeHttpMessage(bt, http_host, url)
+    transaction_encode_http_message = TransactionEncodeHttpMessage(bt, rbuffer, rtype, statuscode, context.flow_path_id)
     transaction_encode_http_message_packet = transaction_encode_http_message.encode()
 
     full_packet = header_packet + transaction_encode_http_message_packet
@@ -485,8 +532,6 @@ class InitMessage(object):
         self.buffer = "dummy_buffer"
 
     def encode(self):
-        #self.msg_type = "init"  # For informing proxy about agent status
-
         init_message_struct = struct.Struct("=c")
         init_message_values = (bytes('^', "utf-8"),)
         init_message_packet = init_message_struct.pack(*init_message_values)
