@@ -48,7 +48,8 @@ class Aisession():
 
 
     def printst(self):
-        path =  "/opt/cavisson/netdiagnostics/logs/"
+        nd_home = os.environ.get('ND_HOME')
+        path = nd_home + '/logs/'
         filename = str(self.sess.decode('ascii'))+"_AI.txt"
         file_to_open = os.path.join(path, filename)
         filenameai = str(self.sess.decode('ascii'))+".txt"
