@@ -10,9 +10,9 @@ from configparser import SafeConfigParser, ConfigParser
 
 try:
     ospath = os.environ.get('ND_HOME')
-    path = ospath + '/config/ndsettings.conf'
+    path = ospath + '/python/config/ndsettings.conf'
 except Exception as e:
-    #print("ND HOME NOT SET")
+    print("ND HOME NOT SET")
     raise e
 
 #print("path of  NDC config : ", path)
@@ -80,14 +80,14 @@ CONFIG_FILE = ''
 APP_NAME = 'CavAgent'
 #TIER_NAME = 'test_tier'
 #NODE_NAME = 'test'
-DIR = ospath + '/CavAgent'
+DIR = ospath + '/python/CavAgent'
 TIER = myvars['tier'].strip('\n')
 SERVER = myvars['server'].strip('\n')
 
 
 # Logging
-LOGS_DIR = ospath+'/CavAgent/logs'
-#print("LOGS_DIR", LOGS_DIR)
+LOGS_DIR = ospath+'/python/CavAgent/logs'
+print("LOGS_DIR", LOGS_DIR)
 
 if "CAV_APP_AGENT_LoggingLevel" in os.environ:
     LOGGING_LEVEL = os.environ.get('CAV_APP_AGENT_LoggingLevel')
