@@ -144,12 +144,14 @@ def command(options, args):
         pythonpath.append(environ['PYTHONPATH'])
 
     environ['PYTHONPATH'] = ':'.join(pythonpath)
+    environ['LD_LIBRARY_PATH'] = "/usr/local/lib"
     environ['LD_PRELOAD'] = "/usr/local/lib/libapr-1.so /usr/local/lib/libwebsockets.so /usr/local/lib/libcrypto.so /usr/local/lib/libssl.so"
     #print('ND_HOME set as ',environ.get('ND_HOME'))
-    print("ND_HOME Environment variable : ", environ.get('ND_HOME'))
+    print("ND_HOME Environment variable :", environ.get('ND_HOME'))
     if environ.get('ND_HOME') is None:
         environ['ND_HOME'] = "/opt/cavisson/netdiagnostics"
-        print("Taking Default ND_HOME path {}", environ.get('ND_HOME'))
+        environ['NDHOME'] = "/opt/cavisson/netdiagnostics"
+        print("Taking Default ND_HOME path",environ.get('ND_HOME'))
     environ['CAV_RUNNING_MODE'] = '0'
     
     if 'config-file' in options:
